/* Original credits for "ping.c"  (but version history removed, they aren't pertient
   as ping.c is largely gutted.  Please blame any ugly code on the author of mturoute,
   not the original author!  -Eli) */

/**
 *			P I N G
 *
 * Using the Internet Control Message Protocol (ICMP) "ECHO" facility to:
 *  - provide network point-to-point statistics concerning packet loss
 *    and round time trip time
 *  - find the route packets take to reach a network host (and display
 *    network statistics about each host)
 *
 * Hack by Francois Gouget (fgouget@free.fr), based on:
 *  - the ping sample provided with the ICMP libraries (author unknown)
 *  - the FreeBSD ping interface
 *  - the FreeBSD traceroute interface
 *  - bing for these comments
 *
 * Comments and bug reports welcome !
 */

#include <stdio.h>
#include <limits.h>
#include <time.h>
#include <winsock2.h>
#include <iphlpapi.h>
#include <ipexport.h>
#include <icmpapi.h>

/* ------------------------------------
 *
 * Application parameters
 *
 * ----------------------------------*/

/**
 * The application name for error messages.
 */
#define PING_NAME               "mturoute"

/**
 * How big is an ICMP header.
 */
#define ICMP_HEADER_SIZE        8
#define IP_HEADER_SIZE			20



/* ------------------------------------
 *
 * Options defaults
 *
 * ----------------------------------*/


/**
 * Send ICMP packets ad infinitum.
 */
#define DEFAULT_OPT_C_VALUE     -1

/**
 * Wait 1 seconde between ICMP packets.
 */
#define DEFAULT_OPT_I_VALUE      1

/**
 * Same as for TCP.
 */
#define DEFAULT_OPT_M_VALUE     30

/**
 * Add 56 bytes of data which makes for a 64 bytes ICMP packet.
 */
#define DEFAULT_OPT_S_VALUE     56

/**
 * Wait for a reply for 5 seconds.
 */
#define DEFAULT_OPT_W_VALUE     3000L

/**
 * Patterns are at most 16 bytes long.
 */
#define OPT_P_MAX_SIZE          16

/** Retry up to three times **/
#define DEFAULT_OPT_R_VALUE 3

/**
 * try to pick a non-crashy default *
 */
#define DEFAULT_PAYLOAD_VALUE 10000


/* ------------------------------------
 *
 * Options flags and values.
 *
 * ----------------------------------*/

/**
 * A bit field for storing boolean options.
 * Note that in some cases the field is not a boolean and this variable 
 * simply stores whether the option has been used while the actual value is 
 * stored elswhere.
 */
unsigned short options=0;

#define F_COUNT         0x0001
#define F_INTERVAL      0x0002
#define F_MAXTTL        0x0004
#define F_NUMERIC       0x0008
#define F_PATTERN       0x0010
#define F_QUIET         0x0020
#define F_DATASIZE      0x0040
#define F_TRACE         0x0080
#define F_TIMEOUT       0x0100
#define F_RANDOMFILL    0x0200
#define F_FRAGMENT		0x0400
#define F_PAYLOAD		0x0800





int opt_c=DEFAULT_OPT_C_VALUE;
int opt_i=DEFAULT_OPT_I_VALUE;
int opt_m=DEFAULT_OPT_M_VALUE;
int opt_p_length;
unsigned char opt_p_pattern[16];
int opt_s=DEFAULT_OPT_S_VALUE;
int opt_w=DEFAULT_OPT_W_VALUE;
int opt_r=DEFAULT_OPT_R_VALUE; /** max retry count **/
int opt_d=0;                   /** debugging **/
int opt_payload=DEFAULT_PAYLOAD_VALUE;


/*
 * Some global variables
 */
HANDLE hIcmp;
unsigned long count;
unsigned short icmp_data_size=0;
unsigned char* icmp_data=NULL;
unsigned char reply_buffer[10000];
LARGE_INTEGER ticks_freq;
struct icmp_echo_reply* icmp_reply;

int control_c_abort=0;


/* ------------------------------------
 *
 * Functions implementing "non-core" functionalities of ping
 *
 * ----------------------------------*/

/**
 * Handles the console events such as CTRL_C_EVENT.
 * We tell the doPing function to stop by setting the count variable to 1. 
 * Because this handler is run in its own thread count must be modified 
 * using InterlockedIncrement and InterlockedDecrement.
 *
 * @param dwCtrlType the code of the console event
 * @return TRUE if the event was handled, FALSE otherwise
 */
BOOL WINAPI ConsoleCtrlHandler(DWORD dwCtrlType)
{
    if (dwCtrlType==CTRL_C_EVENT) {
        if (!(options & F_TRACE)) {
//            count=0;
			control_c_abort = 1;
            return TRUE;
        } else
            return FALSE;
    } else
        return FALSE;
}

/**
 * Converts a string to an IP address. The address may be specified 
 * either as a host name or as the IP address in "dotted" format.
 *
 * @param host_string the host name / address
 * @return the host's IP address if successful, INADDR_NONE otherwise
 */
unsigned int HostString2Addr(char* host_string)
{
    unsigned int host_addr;

    host_addr=inet_addr(host_string);
    if (host_addr==INADDR_NONE) {
        struct hostent* he;

        he=gethostbyname(host_string);
        if (he==NULL) {
            fprintf(stderr,"%s: unknown host %s\n",PING_NAME,host_string);
        } else if (he->h_addrtype!=AF_INET) {
            fprintf(stderr,"%s: host %s is not in the AF_INET domain\n",
                    PING_NAME,host_string);
        } else {
            memcpy(&host_addr, he->h_addr, sizeof(host_addr));
        }
    }
    return host_addr;
}

/**
 * Returns a pointer to a string suitable for representing the host's
 * address. If the options parameter contains the F_NUMERIC option 
 * HostAddr2String will not attempt to get the host name so that the 
 * returned string will only contain the host adress.
 *
 * @param host_addr the host IP address
 * @param options the options value
 * @return a pointer to a static area containing the string to display for 
 *         that host
 */
char* HostAddr2String(int host_addr,int options)
{
    static char host_string[15+1+1+64+1+1];
    char* ip_str;

    ip_str=inet_ntoa(*((struct in_addr*)&host_addr));
    //if (options & F_NUMERIC) {
        strncpy(host_string,ip_str,sizeof(host_string));
    //} else {
    //    struct hostent* he;

    //    he=gethostbyaddr((char*)&host_addr,sizeof(host_addr),AF_INET);
    //    if (he!=NULL)
    //        _snprintf(host_string,sizeof(host_string),"%s (%s)",
    //                  he->h_name,ip_str);
    //    else
    //        _snprintf(host_string,sizeof(host_string),"<unknown> (%s)",
    //                  ip_str);
    //}
    host_string[sizeof(host_string)-1]='\0';
    return host_string;
}

/**
 * Parses the pattern given by the user and places it in the pattern 
 * buffer in a form suitable for further use.
 *
 * @param dynamic 1 if called by doPing or doTrace, 0 otherwise
 * @return 0 if successful, -1 otherwise
 */
int GetPattern(char* pattern)
{
    size_t i,l;
    char subpattern[3];

    l=strlen(pattern);
    if ((l>32) || (l % 2!=0))
        return -1;
    i=0;
    opt_p_length=0;
    subpattern[2]='\0';
    while (pattern[i]!='\0') {
        subpattern[0]=pattern[i++];
        subpattern[1]=pattern[i++];
        if (sscanf(subpattern,"%x",&l)!=1)
            return -1;
        opt_p_pattern[opt_p_length++]=(char)l;
    }
    return 0;
}

/**
 * Builds the data field to be sent with the ICMP packet. The data put 
 * in the packet is either dynamic, e.g. random data, or static (i.e.
 * does not change depending on the packet), e.g. a user specified 
 * pattern. In the first case the data is built on the fly while in the
 * second case it is precomputed.
 * If we try to precompute a packet precomputing must be set to 1, 
 * otherwise it means that we are about to send a packet and that we 
 * want to update the data to be sent.
 *
 * @param precomputing 0 if called by doPing or doTrace, 1 otherwise
 */
void GenerateData(char precomputing)
{
    if (icmp_data_size!=opt_s) {
        icmp_data_size=opt_s;
        icmp_data=realloc(icmp_data,icmp_data_size);
    }

    if (precomputing==1) {
        if (options &  F_PATTERN) {
            int i;

            /* Fill the buffer with the pattern. Since this pattern 
             * does not change we can compute it ahead of time
             */
            i=0;
            while (i<icmp_data_size) {
                memcpy(icmp_data+i,opt_p_pattern,
                       min(opt_p_length,icmp_data_size-i));
                i+=opt_p_length;
            }
        } else if (!(options & F_RANDOMFILL)) {
            int i;

            /* Default case: just put some predictible data in the 
             * packet.
             */
            for (i=0;i<icmp_data_size;i++) {
                icmp_data[i]=(char)i;
            }
        }
    } else {
        if (options & F_RANDOMFILL) {
            int i,r;

            /* Initialise the ICMP payload with random data.
             * Because MAX_RAND is small only the low order byte is usable.
             */
            for (i=0;i<icmp_data_size;i++) {
                r=rand();
                icmp_data[i]=r & 0xff;
            }
        }
    }
}


int doMTU(int target_addr, char *target_string, int MTU)
{
    int status;
    LARGE_INTEGER start,stop;
    double min_rtt,sum_rtt,max_rtt; /* in milliseconds */
    int nb_sent,nb_rtt_samples,nb_rtt_stats;
    struct ip_option_information ip_opts;
	PICMP_ECHO_REPLY pEchoReply;
	int maxRetry;


    /* Prepare the IP options */
    memset(&ip_opts,0,sizeof(ip_opts));
    ip_opts.Ttl=opt_m;
	maxRetry = opt_r;

	/* do not fragment */
	if (!(options & F_FRAGMENT))
	{
		ip_opts.Flags = ip_opts.Flags + IP_FLAG_DF;
	}



    if (options & F_COUNT)
        count=opt_c;
    else
        count=INT_MAX;


    nb_sent=nb_rtt_samples=nb_rtt_stats=0;
    min_rtt=86400000.0; /* One day */
    sum_rtt=max_rtt=0.0;

    while ((InterlockedDecrement(&count)!=-1) && (maxRetry > 0)) {

        GenerateData(0);

        QueryPerformanceCounter(&start);


        status=IcmpSendEcho(hIcmp,
                            target_addr,
                            (LPVOID)icmp_data,
                            //icmp_data_size,
							MTU,
                            &ip_opts,
                            reply_buffer,
                            sizeof(reply_buffer),
                            opt_w);

		pEchoReply = (PICMP_ECHO_REPLY)reply_buffer;
		QueryPerformanceCounter(&stop);
		if (status == 0) {
			status=GetLastError();
			if (opt_d && (status > 1)) printf("GetLastError called after IcmpSendEcho returned %d\n",status); 
		} else {
			if (opt_d && (status > 1)) { printf("IcmpSendEcho returned error %d\n",status); }
		}
		if (status <= 1) {
			status=pEchoReply -> Status;
			if (opt_d && status) printf("IcmpSendEcho reply buffer contains %d\n",status);
		}
		if (status == IP_PACKET_TOO_BIG) return 1;
		if (status == IP_REQ_TIMED_OUT) { printf("."); maxRetry --; continue; }
		if (status == IP_DEST_NET_UNREACHABLE) { printf("u"); return 0; }
		return status;
	}
	return 1;
}

void output_parameters()
{
	/* do not fragment */
	if (!(options & F_FRAGMENT))
	{
		printf("* ICMP Fragmentation is not permitted. *\n");
	}
	else
	{
		printf("* ICMP Fragmentation is permitted. *\n");
	}

	printf("* Maximum payload is %d bytes. *\n", opt_payload );
}


int doSingleMTU(int target_addr, char *target_string, int display) {

	int min_mtu = 92;
	int max_mtu;
	int orig_max_mtu;
	int orig_min_mtu;

	//int max_mtu = 65280;
//	int max_mtu = 60000;

	int current_check;

	int success;

	//printf("doAllMTU starting\n");

	int counter = 0;
	int last_checked = 0;

	int we_matched = 0;

    max_mtu = opt_payload;

	orig_min_mtu = min_mtu;
	orig_max_mtu = max_mtu;

	//while (counter < 10) {
	if (display == 1)
	{
		output_parameters();
	}


	while (min_mtu != max_mtu && we_matched == 0) {

		/* break out if control-c is pressed */
		if (control_c_abort == 1) {
			fprintf(stderr,"^C\n");
			return 1;
		}

		current_check = (max_mtu + min_mtu) / 2;
		//printf("trying a packet size of %d...\n", current_check);

		success = doMTU( target_addr, target_string, current_check );
		//printf("got out of doMTU");

		if (success == 0) {
			// mtu worked
			if (display == 1)
			{
				printf("+ ICMP payload of %d bytes succeeded.\n", current_check);
			}
			else {
				printf("+");
			}
			min_mtu = current_check;
		}
		else {
			// mtu failed
			if (display == 1)
			{
				printf("- ICMP payload of %d bytes failed..\n", current_check);
			}
			else {
				printf("-");
			}
			max_mtu = current_check;
		}
		if (last_checked == current_check)
		{
			we_matched = 1;
		}
		last_checked = current_check;
		Sleep(100 * opt_i);
		counter++;

	}
	if (last_checked +1 >= orig_max_mtu && display == 1)
	{
		printf("Estimated Max: %d bytes or higher.", max_mtu);
	}
	else if (last_checked - 1 <= orig_min_mtu && display == 1)
	{
		printf("Target is not responding to pings.");
	}
	else
	{
		if (display == 1)
		{
				/* do not fragment */
			if (!(options & F_FRAGMENT))
			{
				printf ("Path MTU: %d bytes.\n", last_checked + ICMP_HEADER_SIZE + IP_HEADER_SIZE);
			}
			else {
				printf ("Max ping: %d bytes.\n", last_checked + ICMP_HEADER_SIZE + IP_HEADER_SIZE);
			}
		}
	}
	return last_checked + ICMP_HEADER_SIZE + IP_HEADER_SIZE;
}




/**
 * Sends ICMP echo Request packets with increasing TTLs to discover 
 * the list of hosts the packets are routed through on their way to  
 * the target host.
 *
 * @param target_addr internet address of the target host
 * @param target_string string to display for that host
 */
void doManyMTU(unsigned int target_addr,char* target_string)
{
    int status;
    int hop,probe,nb_probes;
    unsigned int gateway_addr;
    struct ip_option_information ip_opts;
	int foundMTU;
	//int multiple_gateways;

    printf("mturoute to %s, %d hops max, variable sized packets\n",
           target_string,opt_m,ICMP_HEADER_SIZE+opt_s);

	output_parameters();

    /* Prepare the IP options */
    memset(&ip_opts,0,sizeof(ip_opts));

    nb_probes=(opt_c>0?opt_c:3);
    for (hop=1;hop<opt_m;hop++) {
        double rtt,min_rtt,sum_rtt,max_rtt;
        int nb_rtt_samples,nb_rtt_stats;
        LARGE_INTEGER start,stop;

        printf("%2d  ",hop);
        ip_opts.Ttl=hop;
        gateway_addr=INADDR_NONE;

        /* No we must send some probes with the specified TTL */
        nb_rtt_samples=nb_rtt_stats=0;
        min_rtt=86400000.0; /* One day */
        sum_rtt=max_rtt=0.0;
        for (probe=0;probe<nb_probes;probe++) {
            GenerateData(0);
            QueryPerformanceCounter(&start);
            status=IcmpSendEcho(hIcmp,
                                target_addr,
                                (LPVOID)icmp_data,
                                icmp_data_size,
                                &ip_opts,
                                reply_buffer,
                                sizeof(reply_buffer),
                                opt_w);
            QueryPerformanceCounter(&stop);

            /* Report this probe's result */
            if (status==0) {
                status=GetLastError();
                if (status!=IP_REQ_TIMED_OUT)
                    fprintf(stderr,"An error (%d) occurred while sending the ICMP Echo Request to %s\n",
                            status,target_string);
                //if (!(options & F_QUIET))
                //    printf("  * ms  ");
            } else {
                if (gateway_addr==INADDR_NONE) {
                    gateway_addr=icmp_reply->Address;
					
					//printf("firing off a doSingleMTU at %s\n", gateway_addr);
					foundMTU = doSingleMTU(gateway_addr, target_string, 0);
					printf("  host: %s  max: %d bytes \n",HostAddr2String(gateway_addr,options), foundMTU);
					//multiple_gateways = 0;

					//printf("Hop %s had MTU of %d\n", target_string, foundMTU);
                } else if (gateway_addr!=icmp_reply->Address) {
//                    fprintf(stderr,"\nAdditional gateways answered for hop %d: %s\n",
//                            hop,HostAddr2String(icmp_reply->Address,options));
						printf("*%d (An additional device responded for %s.)\n", hop, HostAddr2String(gateway_addr, options));
						//foundMTU = doSingleMTU(gateway_addr, target_string, 0);
						//printf("  host: %s  max: %d bytes \n",HostAddr2String(gateway_addr,options), foundMTU);
					//multiple_gateways = 1;
                }

                /* compute the RTT, see doPing for explanations */
                if ((ticks_freq.QuadPart==0) ||
                    ((rtt=((double)((stop.QuadPart-start.QuadPart)*1000))/ticks_freq.QuadPart)>
                     (double)(icmp_reply->RoundTripTime+2))) {
                    rtt=(double)(icmp_reply->RoundTripTime+1);
                }

                if (options & F_QUIET) {
                    if (rtt<(double)(icmp_reply->RoundTripTime+2)) {
                        /* Update the statistics */
                        if (rtt<min_rtt)
                            min_rtt=rtt;
                        if (rtt>max_rtt)
                            max_rtt=rtt;
                        sum_rtt+=rtt;
                        nb_rtt_stats++;
                    }
                    nb_rtt_samples++;
                } //else {
                  //  printf("%.3f ms  ",rtt);
                //}
            }
        }
        /* Display statistics for that host */
//        if (options & F_QUIET) {
//            printf("sent %d received %d loss %d%% ",
//                   nb_probes,nb_rtt_samples,(nb_probes-nb_rtt_samples)*100/nb_probes);
//            if (nb_rtt_samples>0) {
//                printf("rtt %.3f/%.3f/%.3f ms  ",
//                       min_rtt,((float)sum_rtt)/nb_rtt_stats,max_rtt);
//            } else {
//                printf("rtt */*/* ms  ");
//            }
//         }

        //if (gateway_addr!=INADDR_NONE /*&& multiple_gateways == 0*/)
        //    printf("  host: %s  max: %d bytes \n",HostAddr2String(gateway_addr,options), foundMTU);
        //else
        //    printf("\n");
        if (gateway_addr==target_addr)
            break;
//        Sleep(100*opt_i);
    }
}
/* ------------------------------------
 *
 * Main program
 *
 * ----------------------------------*/

/**
 * Parses the ping parameters and calls doPing or doTrace as appropriate.
 * For the tool usage, see comments at the beginning.
 *
 * @param argc number of command-line options
 * @param argv command line options
 * @return 0 if sucessful, 2 for a usage error and 1 otherwise
 */
int main(int argc, char** argv)
{
    int retcode;
    WSADATA wsaData;
    char** arg;

	int tmp;

    if (argc<2)
        goto usage;

    /* Initialisation */
    if (WSAStartup(MAKEWORD(1,1), &wsaData)) {
        fprintf(stderr,"%s: You must use Winsock 1.1 or compatible\n",PING_NAME);
        goto error;
    }
    hIcmp = IcmpCreateFile();
    srand((int) time(NULL));
    /* Give a higher priority so that ping has better 
     * chances not to be delayed when measuring the RTT.
     */
    SetPriorityClass(GetCurrentProcess(),HIGH_PRIORITY_CLASS);
    QueryPerformanceFrequency(&ticks_freq);
    icmp_reply=(struct icmp_echo_reply*)reply_buffer;

    /* Install the ^C handler */
    SetConsoleCtrlHandler(&ConsoleCtrlHandler,TRUE);

    /* Parse the parameters and do the pinging and traceing */
    retcode=0;
    arg=argv+1;
    GenerateData(1);
    while (*arg!=NULL) {
        if ((*arg)[0]=='-') {
            /* Change some option value */
            switch ((*arg)[1]) {
            //case 'c':
                /* Sets the number of messages to send to the 
                 * target host. Only valid in ping mode.
                 */
            //    arg++;
            //    if (sscanf(*arg,"%d",&opt_c)!=1)
            //        goto usage;
            //    if (opt_c<0)
            //        opt_c=DEFAULT_OPT_C_VALUE;
            //    options|=F_COUNT;
             //   break;
            case 'd':
                 /* turns on debugging of ICMP status/failure codes */
				 opt_d ++; break;
            //case 'f':
                /* Causes ping to flood the target host sending EchoRequest 
                 * at a rate of 100 per second or as fast as they come back 
                 * whichever is more.
                 * => Cannot be implemented with this ICMP interface because 
                 *    IcmpSendEchoRequest will block until it receives an 
                 *    answer (maybe using multiple thread would provide an 
                 *    awkward solution).
                 *    Note though that setting the -i option to 0 is roughly 
                 *    equivalent on fast networks.
                 */
            //    break;
            case 'i':
                /* Sets the interval between two echo requests. */
                arg++;
                if (sscanf(*arg,"%d",&opt_i)!=1)
                    goto usage;
                if (opt_i<0)
                    opt_i=DEFAULT_OPT_I_VALUE;
                options|=F_INTERVAL;
                break;
            case 'r':
                /* Sets the retry count. */
                arg++;
                if (sscanf(*arg,"%d",&opt_r)!=1)
                    goto usage;
                if (opt_r<0)
                    opt_r=DEFAULT_OPT_R_VALUE;
                break;
            //case 'l':
                /* Causes ping to send the specified number of packets before
                 * waiting.
                 * => Cannot be implemented with this ICMP interface because 
                 *    IcmpSendEchoRequest will block until it receives an 
                 *    answer (maybe using multiple thread would provide an 
                 *    awkward solution).
                 */
            //    break;
            //case 'm':
            //    /* Sets the TTL to use.
            //     */
            //    arg++;
            //    if (sscanf(*arg,"%d",&opt_m)!=1)
            //        goto usage;
            //    if (opt_m<0)
            //        opt_m=DEFAULT_OPT_M_VALUE;
            //    options|=F_MAXTTL;
            //    break;
            //case 'n':
                /* Do not attempt to translate symbolic names for host 
                 * addresses
                 */
            //    options|=F_NUMERIC;
            //    break;
            //case 'p':
                /* Pattern to use to fill the packets
                 */
            //    arg++;
            //    if (GetPattern(*arg)<0) {
            //        fprintf(stderr,"%s: incorrect pattern\n",PING_NAME);
            //        goto usage;
            //    }
            //    options|=F_PATTERN;
            //    GenerateData(1);
            //    break;
            //case 'q':
                /* Quiet option: only display a summary at the end.
                 * Only valid in ping mode.
                 */
            //    options|=F_QUIET;
            //    break;
            //case 'R':
                /*
                 * Record route.
                 * => Could be implemented by manually adding the proper 
                 *    IP option fields and by manually decoding the answer.
                 */
            //   break;
            //case 'r':
                /* Bypass normal routing.
                 * => Not implemented.
                 *    I'm not sure whether this could be implemented
                 */
            //    break;
            //case 's':
                /*
                 * Set the packet size.
                 */
            //    arg++;
            //    if (sscanf(*arg,"%d",&opt_s)!=1)
            //        goto usage;
            //    if (opt_s<0)
            //        opt_s=DEFAULT_OPT_S_VALUE;
            //    options|=F_DATASIZE;
            //    GenerateData(1);
            //    break;
            case 'm':
                /*
                 * Set the maximum payload size
                 */
                arg++;
                if (sscanf(*arg,"%d",&opt_payload)!=1)
                    goto usage;
				if (opt_payload <= 92 || opt_payload >= 65280)
				{
					fprintf(stderr, "Maximum payload must be between 92 and 65280.\n\n");
					goto end;
				}
                if (opt_payload<0)
                    opt_payload=DEFAULT_PAYLOAD_VALUE;
                options|=F_PAYLOAD;
                GenerateData(1);
                break;
            case 't':
                /*
                 * Switch to traceroute mode.
                 */
                options|=F_TRACE;
                break;
			case 'f':
				/*
				 * Switch to nofragment mode.
				 */
				 options|=F_FRAGMENT;
				 break;
            //case 'v':
                /*
                 * Print messages other than EchoResponse
                 * => Not implemented
                 *    The implementation requires parsing the IP options data 
                 *    contained in the icmp_reply
                 */
            //    break;
            case 'w':
                /*
                 * Sets the time to wait for a response to an Echo Request
                 */
                arg++;
                if (sscanf(*arg,"%d",&opt_w)!=1)
                    goto usage;
                if (opt_w<0)
                    opt_w=DEFAULT_OPT_W_VALUE;
                options|=F_TIMEOUT;
                break;
            //case 'z':
                /*
                 * Fills the ICMP packets with random data
                 */
            //    options=(options & ~F_PATTERN) | F_RANDOMFILL;
            //    break;
            }
        } else {
            unsigned int host_addr;

            /* Get the target host address */
            host_addr=HostString2Addr(*arg);
            if (host_addr==INADDR_NONE)
                retcode=1;
			//else
			//	doAllMTU(host_addr,*arg);
			 //   tmp = doSingleMTU(host_addr,*arg,1);
            else if (options & F_TRACE)
                doManyMTU(host_addr,*arg);
				//doTrace(host_addr,*arg);
            else
                tmp = doSingleMTU(host_addr,*arg, 1);
        }
        arg++;
    }

    /* Do some cleanup */
    IcmpCloseHandle( hIcmp );
    WSACleanup();
    free(icmp_data);

end:
    return retcode;

error:
    retcode=1;
    goto end;

usage:
	fprintf(stderr,"********************************************************\n");
	fprintf(stderr,"* mturoute.exe - written by Eli Fulkerson, June 2005   *\n");
	fprintf(stderr,"*                fixes by Ivan Pepelnjak, October 2007 *\n");
	fprintf(stderr,"* Please visit http://www.elifulkerson.com for updates.*\n");
	fprintf(stderr,"*       http://ioshints.blogspot.com/search?q=mturoute *\n");
	fprintf(stderr,"********************************************************\n");
	fprintf(stderr, "\n");
    fprintf(stderr,"Usage: mturoute [-t] [-f] [-m MAX_PAYLOAD_SIZE] host \n");
    fprintf(stderr,"Flags:\n");
	fprintf(stderr,"   -t : Toggles 'traceroute' mode.  (Default is off)\n");
	fprintf(stderr,"   -f : Allow fragmentation.  This will return the max ping size that the\n");
	fprintf(stderr,"        target host will respond too, but not necessarily the MTU.\n");
	fprintf(stderr,"   -w : Sets the time to wait for a response to an Echo Request.\n");
	fprintf(stderr,"   -r : Sets the maximum number of probe retries on timeout (default = 3).\n");
	fprintf(stderr,"   -i : Sets the interval between two echo requests.\n");
	fprintf(stderr,"   -d : Increases the debugging level. Reports ICMP status/failures.\n");
	fprintf(stderr,"   -m : Sets a maximum payload size to test. (Default is 10000)\n");
	fprintf(stderr,"        Large payload sizes (30000+) are known to crash the utility. What\n");
	fprintf(stderr,"        these sizes are varies based on the windows install.  If you have\n");
	fprintf(stderr,"        trouble, try setting a lower maximum size.\n");
	fprintf(stderr,"\n");
	fprintf(stderr,"Warning:  This utility generates a lot of ICMP traffic.\n");
    retcode=2;
    goto end;

}
